@@ -1,8 +1,8 @@
 #="is rw" als Sub-Trait
 sub storage is rw {...}
 storage() = 5;
-#="is constant"  ist der Standard
-sub func( $arg is constant ) { $arg = 123; } # geht nicht!
+#="is readonly"  ist der Standard
+sub func( $arg is readonly ) { $arg = 123; } # geht nicht!
 #="is rw"  Parameter-Trait
 sub swap (*@_ is rw) { @_[0,1] = @_[1,0] }
 #="is copy"
