@@ -36,7 +36,7 @@ sub max (Array @x) returns Int is export{
     $max.int
 }    
 
-sub whisper_about (Int $sizeof_crowd, Array ?@x) returns Hash is
+sub whisper_about (Int $sizeof_crowd, Array @x?) returns Hash is
 export{
     my %terms;
     for 1 .. $sizeof_crowd {
@@ -69,18 +69,15 @@ sub append_last ($x,$string is rw,$pass){
         pick (
             "$x",
             "is $x",
-        ); 
+        );
     }else{
       "$string $x";
     }
 }
 
-sub matchval ($x,Hash %x,Int ?$cap) returns Array is export{
-    my @matches; 
-    my %copy = hash %x; #XXX Bug
-    # say %x.ref    ; # List  
-    # say %copy.ref ; # Hash
-    for %copy.kv->$key,$value{
+sub matchval ($x,Hash %x,Int $cap?) returns Array is export{
+    my @matches;
+    for %x.kv->$key,$value{
         if $value eq $x { push @matches,$key }  
         if $cap && +@matches == $cap { return @matches }
     }
