@@ -31,6 +31,21 @@ use strict;
   }
 }
 
+# This class does not have a Haskell equivalent.
+{
+  package PIL::NamedPair;
+  our @ISA = qw<PIL::PVal>;
+
+  sub as_js {
+    my $key = ref $_[0]->{key}
+      ? sprintf "PIL2JS.cps2normal(_26main_3a_3aprefix_3a_7e.FETCH(), [PIL2JS.Context.ItemAny, %s]).toNative()", $_[0]->{key}->as_js
+      : PIL::doublequote($_[0]->{key});
+    sprintf "new PIL2JS.Box.Constant(new PIL2JS.NamedPair(%s, %s))",
+      $key,
+      $_[0]->{value}->as_js;
+  }
+}
+
 # Basic datatypes (VInt, VRat, VStr, VUndef). Important: Box the things with
 # new PIL2JS.Box.Constant!
 {
@@ -59,7 +74,7 @@ use strict;
   package PIL::VRat;
   our @ISA = qw<PIL::PVal>;
 
-  sub as_js { "new PIL2JS.Box.Constant($_[0]->[0])" }
+  sub as_js { "new PIL2JS.Box.Constant($_[0]->[0][0]/$_[0]->[0][1])" }
 }
 
 {
