@@ -6,9 +6,13 @@ method JS::Root::defined($a:) {
   )')($a);
 }
 
-method JS::Root::not(*@a:) { @a.elems == 0 ?? undef !! !@a[0] } # XXX correct?
+method JS::Root::not(*@a:) { @a.elems == 0 ?? () # should be: want.Scalar ?? undef !! ()
+                                           !! !@a[0] } # XXX correct?
 method JS::Root::true($a:) { ?$a }
 
+sub bool::true is primitive { ?1 }
+sub bool::false is primitive { ?0 }
+
 sub prefix:<!>($a) is primitive { $a ?? ?0 !! ?1 }
 
 sub infix:<^^>   ($a, $b) is primitive {
