@@ -1,6 +1,6 @@
 sub PIL2JS::Internals::new_proxy (
-  Code +$FETCH = { die "No FETCH routine supplied" },
-  Code +$STORE = { die "No STORE routine supplied" },
+  Code :$FETCH = { die "No FETCH routine supplied" },
+  Code :$STORE = { die "No STORE routine supplied" },
 ) is rw {
   JS::inline('new PIL2JS.Box.Constant(function (args) {
     var fetch = args[1].FETCH(), store = args[2].FETCH(), cc = args.pop();
