@@ -3,10 +3,12 @@
 #ifndef PUGS_HAVE_PERL5
 module Pugs.Embed.Perl5 where
 import Foreign.C.Types
+import Data.Typeable
 
 type PerlInterpreter = ()
-type PerlSV = ()
-type PugsVal = ()
+data PerlSV = MkPerlSV -- phantom type
+    deriving (Show, Eq, Ord, Typeable)
+type PugsVal = PerlSV
 
 constFail :: a -> IO b
 constFail = const $ fail "perl5 not embedded"
@@ -36,10 +38,10 @@ svToVal :: PerlSV -> IO a
 svToVal = constFail
 
 mkVal :: (Show a) => a -> IO PugsVal
-mkVal = const $ return ()
+mkVal = constFail
 
 mkValRef :: a -> IO PerlSV
-mkValRef = const $ return ()
+mkValRef = constFail
 
 vstrToSV :: String -> IO PerlSV
 vstrToSV = constFail
@@ -64,14 +66,15 @@ mkSV = id
 
 #else
 
-{-# INCLUDE <p5embed.h> #-}
-{-# INCLUDE <pugsembed.h> #-}
+{-# INCLUDE "../../perl5/p5embed.h" #-}
+{-# INCLUDE "../../perl5/pugsembed.h" #-}
 
 module Pugs.Embed.Perl5 where
 import Pugs.Internals
 import Foreign
 import Foreign.C.Types
 import Foreign.C.String
+import Data.IORef (modifyIORef)
 
 type PerlInterpreter = Ptr ()
 type PerlSV = Ptr ()
@@ -89,41 +92,41 @@ foreign import ccall "perl.h perl_free"
     perl_free :: PerlInterpreter -> IO ()
 foreign import ccall "perl.h boot_DynaLoader"
     boot_DynaLoader :: Ptr () -> IO ()
-foreign import ccall "p5embed.h perl5_finalize"
+foreign import ccall "../../perl5/p5embed.h perl5_finalize"
     perl5_finalize :: PerlSV -> IO ()
-foreign import ccall "p5embed.h perl5_SvPV"
+foreign import ccall "../../perl5/p5embed.h perl5_SvPV"
     perl5_SvPV :: PerlSV -> IO CString
-foreign import ccall "p5embed.h perl5_SvIV"
+foreign import ccall "../../perl5/p5embed.h perl5_SvIV"
     perl5_SvIV :: PerlSV -> IO CInt
-foreign import ccall "p5embed.h perl5_SvNV"
+foreign import ccall "../../perl5/p5embed.h perl5_SvNV"
     perl5_SvNV :: PerlSV -> IO CDouble
-foreign import ccall "p5embed.h perl5_SvTRUE"
+foreign import ccall "../../perl5/p5embed.h perl5_SvTRUE"
     perl5_SvTRUE :: PerlSV -> IO Bool
-foreign import ccall "p5embed.h perl5_newSVpv"
+foreign import ccall "../../perl5/p5embed.h perl5_newSVpv"
     perl5_newSVpv :: CString -> IO PerlSV
-foreign import ccall "p5embed.h perl5_newSViv"
+foreign import ccall "../../perl5/p5embed.h perl5_newSViv"
     perl5_newSViv :: CInt -> IO PerlSV
-foreign import ccall "p5embed.h perl5_newSVnv"
+foreign import ccall "../../perl5/p5embed.h perl5_newSVnv"
     perl5_newSVnv :: CDouble -> IO PerlSV
-foreign import ccall "p5embed.h perl5_get_sv"
+foreign import ccall "../../perl5/p5embed.h perl5_get_sv"
     perl5_get_sv :: CString -> IO PerlSV
-foreign import ccall "p5embed.h perl5_apply"
+foreign import ccall "../../perl5/p5embed.h perl5_apply"
     perl5_apply :: PerlSV -> PerlSV -> Ptr PerlSV -> PugsVal -> CInt -> IO (Ptr PerlSV)
-foreign import ccall "p5embed.h perl5_can"
+foreign import ccall "../../perl5/p5embed.h perl5_can"
     perl5_can :: PerlSV -> CString -> IO Bool
-foreign import ccall "p5embed.h perl5_eval"
+foreign import ccall "../../perl5/p5embed.h perl5_eval"
     perl5_eval :: CString -> PugsVal -> CInt -> IO PerlSV
-foreign import ccall "p5embed.h perl5_init"
+foreign import ccall "../../perl5/p5embed.h perl5_init"
     perl5_init :: CInt -> Ptr CString -> IO PerlInterpreter
 
-foreign import ccall "pugsembed.h pugs_getenv"
+foreign import ccall "../../perl5/pugsembed.h pugs_getenv"
     pugs_getenv :: IO PugsVal
-foreign import ccall "pugsembed.h pugs_setenv"
+foreign import ccall "../../perl5/pugsembed.h pugs_setenv"
     pugs_setenv :: PugsVal -> IO ()
 
-foreign import ccall "pugsembed.h pugs_SvToVal"
+foreign import ccall "../../perl5/pugsembed.h pugs_SvToVal"
     pugs_SvToVal :: PerlSV -> IO PugsVal
-foreign import ccall "pugsembed.h pugs_MkValRef"
+foreign import ccall "../../perl5/pugsembed.h pugs_MkValRef"
     pugs_MkValRef :: PugsVal -> IO PerlSV
 
 initPerl5 :: (Show a) => String -> Maybe a -> IO PerlInterpreter
@@ -134,6 +137,7 @@ initPerl5 str env = do
             case env of
                 Just val -> pugs_setenv =<< mkVal val
                 Nothing -> return ()
+            modifyIORef _GlobalFinalizer (>> perl_free interp)
             return interp
 
 mkVal :: (Show a) => a -> IO PugsVal
