@@ -1,5 +1,5 @@
 {-# OPTIONS_GHC -fglasgow-exts #-}
-{-# OPTIONS_GHC -#include "UnicodeC.h" #-}
+{-# OPTIONS_GHC -#include "../UnicodeC.h" #-}
 
 {-|
     Pretty printing for various data structures.
@@ -20,6 +20,7 @@ import Pugs.Types
 import Pugs.AST
 import Text.PrettyPrint
 import qualified Data.Set as Set
+import qualified Data.Map as Map
 
 defaultIndent :: Int
 defaultIndent = 2
@@ -39,7 +40,7 @@ instance Pretty Exp where
     format (App sub invs args) = text "App" <+> parens (format sub) <+> parens (nest defaultIndent $ vcat (punctuate (text ", ") (map format $ maybeToList invs ++ args)))
     format (Sym scope name exp) = text "Sym" <+> text (show scope) <+> format name $+$ format exp
     format (Pad scope pad exp) = text "Pad" <+> text (show scope) <+> format pad $+$ format exp
-    format (Pos _ exp) = format exp
+    format (Ann _ exp) = format exp
     format x = text $ show x
 
 instance Pretty (TVar Bool, TVar VRef) where
@@ -70,7 +71,7 @@ instance Pretty Env where
     format x = doubleBraces $ nest defaultIndent (format $ envBody x) 
 
 instance Pretty (Val, Val) where
-    format (x, y) = format x <+> text "=>" <+> format y
+    format (x, y) = hang (format x <+> text "=>") defaultIndent (format y)
 
 instance Pretty (Exp, SourcePos) where
     format (x, _) = format x 
@@ -81,6 +82,39 @@ instance Pretty (TVar VRef) where
 instance Pretty VRef where
     format x = braces $ text $ "ref:" ++ show x
 
+instance Pretty VMatch where
+    format m = joinList (text ", ")
+        [ form ("ok",        matchOk)
+        , form ("from",      matchFrom)
+        , form ("to",        matchTo)
+        , form ("str",       matchStr)
+        , form ("sub_pos",   matchSubPos)
+        , form ("sub_named", matchSubNamed)
+        ]
+        where
+        form :: Pretty a => (String, VMatch -> a) -> Doc
+        form (s, f) = hang (text s <+> text "=>") defaultIndent (format $ f m)
+
+instance Pretty Bool where
+    format x = text $ if x then "bool::true" else "bool::false"
+
+instance Pretty Int where
+    format i = int i
+
+instance Pretty VList where
+    format [x] = parens $ format x <> text ","
+    format xs
+        | not . null . (drop 100) $ xs = parens $ (format (head xs) <+> text ", ...")
+        | otherwise = parens $ (joinList $ text ", ") (map format xs)
+
+instance Pretty VHash where
+    format x = cat
+        [ text "{"
+        , nest defaultIndent . joinList (text ", ") $
+            [ format (VStr k, v) | (k, v) <- Map.toList x ]
+        , text "}"
+        ]
+
 instance Pretty Val where
     format (VJunc j) = parens $ joinList mark items 
         where
@@ -93,16 +127,19 @@ instance Pretty Val where
             JAll  -> text " & "
             JOne  -> text " ^ "
             JNone -> text " ! "
-    format (VBool x) = if x then text "bool::true" else text "bool::false"
-    format (VNum x) = if x == 1/0 then text "Inf" else text $ show x
+    format (VBool x) = format x
+    format (VNum x) = if x == 1/0
+                         then text "Inf"
+                         else if x == -1/0 
+                                then text "-Inf"
+                                else text $ show x
     format (VInt x) = integer x
-    format (VStr x) = text $ "'" ++ encodeUTF8 (concatMap quoted x) ++ "'"
-    format (VRat x) = text $ showRat x
+    format (VStr x) = text $ "\"" ++ encodeUTF8 (concatMap quoted x) ++ "\""
+    format (VRat x) = text $ showTrueRat x
     format (VComplex x) = text $ show x
     format (VControl (ControlEnv _)) = text "<env>"
     format (VControl x) = text $ show x
     format (VProcess x) = text $ show x
-    format (VMatch x) = text $ show x
     format (VOpaque (MkOpaque x)) = braces $ text $ "obj:" ++ show x
 {-
     format (VRef (VList x))
@@ -111,18 +148,18 @@ instance Pretty Val where
         | otherwise = brackets $ cat $ (punctuate $ text ", ") (map format x)
 -}
     format (VRef x) = format x
-    format (VList x)
-        | not . null . (drop 100) $ x
-        = parens $ (format (head x) <+> text ", ...")
-        | otherwise = parens $ (joinList $ text ", ") (map format x)
+    format (VList x) = format x
     format (VCode _) = text "sub {...}"
     format (VBlock _) = text "{...}"
     format (VError x posList)
         -- Is this correct? Does this work on win32, too?
-        | last x == '\n' = text . init $ x
+        | last s == '\n' = text . init $ s
         | otherwise      = text "***" <+>
-            (vcat (map text $ split "\n" x) $+$ (text "at" <+> vcat (map format $ reverse posList)))
+            (vcat (map text $ split "\n" s) $+$ (text "at" <+> vcat (map format $ reverse posList)))
         where
+        s = case x of
+              VStr s' -> s'
+              _       -> pretty x
 --  format (VArray x) = format (VList $ Array.elems x)
 --  format (VHash h) = braces $ (joinList $ text ", ") $
 --      [ format (VStr k, v) | (k, v) <- Map.toList h ]
@@ -136,12 +173,28 @@ instance Pretty Val where
     format (VSubst _) = text $ "{subst}"
     format (VType t) = text $ "::" ++ showType t
     format (VObject o) = text $ "{obj:" ++ showType (objType o) ++ "}"
+    format (VMatch m) = cat
+        [ text "Match.new("
+        , nest defaultIndent $ format m
+        , text ")"
+        ]
     format (PerlSV _) = text $ "{obj-perl5}"
     format VUndef = text $ "undef"
 
 quoted :: Char -> String
 quoted '\'' = "\\'"
 quoted '\\' = "\\\\"
+quoted '"'  = "\\\""
+quoted '{'  = "\\{"
+quoted '\t' = "\\t"
+quoted '\r' = "\\r"
+quoted '\n' = "\\n"
+quoted '$'  = "\\$"
+quoted '@'  = "\\@"
+quoted '%'  = "\\%"
+quoted '&'  = "\\&"
+quoted '^'  = "\\^"
+quoted x | x < ' ' || x > '~' = "\\d[" ++ show (ord x) ++ "]"
 quoted x = [x]
 
 doubleBraces :: Doc -> Doc
@@ -151,5 +204,5 @@ joinList :: Doc -> [Doc] -> Doc
 joinList x y = cat $ punctuate x y
 
 pretty :: Pretty a => a -> String
-pretty a = render $ format a 
+pretty a = render $ format a
 
