@@ -34,8 +34,8 @@ data SourcePos      = SourcePos {sourceName   :: SourceName,
                                  sourceLine   :: !Line,
                                  sourceColumn :: !Column}
                                                
-		     deriving (Eq,Ord)
-		
+                     deriving (Eq,Ord)
+                
 
 newPos :: SourceName -> Line -> Column -> SourcePos
 newPos sourceName line column
@@ -82,5 +82,5 @@ forcePos pos@(SourcePos _ line column)
 -----------------------------------------------------------                                                 
 instance Show SourcePos where
   show (SourcePos name line column) =
-    unwords ["SourcePos", show name, show line, show column]
+    show name ++ " line " ++ show line ++ " column " ++ show column
 
