@@ -47,7 +47,8 @@ sub get_config {
     add_path(installsitearch    => $config); 
     add_path(installsitelib     => $config); 
 
-    $config->{pugspath} =
+    $config->{perl5path} = $^X;
+    $config->{pugspath}  =
       File::Spec->catfile($config->{bin}, "pugs$config->{exe_ext}");
 
     ($config->{file_sep}) =
@@ -116,7 +117,7 @@ __DATA__
 {-
     *** NOTE ***
     DO NOT EDIT THIS FILE.
-    This module is generated by util/generate_config.
+    This module is generated by util/PugsConfig.pm.
 -}
 
 {-|
