@@ -40,16 +40,20 @@ $ENV{PATH} = "." . $Config{path_sep} . $ENV{PATH};
 $ENV{HARNESS_PERL}  = "./pugs $optional_args";
 $ENV{HARNESS_PERL}  = "$^X $FindBin::Bin/../perl5/PIL2JS/pugs-smokejs.pl ./pugs $optional_args"
     if $ENV{PUGS_RUNTIME} and $ENV{PUGS_RUNTIME} eq 'JS';
+# XXX: hack to be identified by smokeserv
+$ENV{HARNESS_PERL}  = "$^X -I/tmp/JSPERL5 $FindBin::Bin/../perl5/PIL2JS/pugs-smokejs.pl ./pugs $optional_args"
+    if $ENV{PUGS_RUNTIME} and $ENV{PUGS_RUNTIME} eq 'JSPERL5';
 $ENV{HARNESS_PERL}  = "$^X $FindBin::Bin/../perl5/PIL-Run/pugs-p5.pl"
     if $ENV{PUGS_RUNTIME} and $ENV{PUGS_RUNTIME} eq 'PERL5';
 
 $ENV{PERL6LIB}      = join $Config{path_sep},
-        qw<ext/Test/lib blib6/lib>, $ENV{PERL6LIB};
+        qw<ext/Test/lib blib6/lib>, $ENV{PERL6LIB}||"";
 
 my @yaml_harness_args;
 push(@yaml_harness_args,'--exclude','Disabled,^ext\b')
-    if $ENV{PUGS_RUNTIME} and ($ENV{PUGS_RUNTIME} eq 'JS' or
-                   $ENV{PUGS_RUNTIME} eq 'PERL5');
+    if $ENV{PUGS_SMOKE_EXCLUDE_EXT}
+        or ($ENV{PUGS_RUNTIME} and ($ENV{PUGS_RUNTIME} eq 'JS' or
+                   $ENV{PUGS_RUNTIME} eq 'PERL5' or $ENV{PUGS_RUNTIME} eq 'JSPERL5'));
 
 sub make { return `$Config{make} @_` };
 my $dev_null = File::Spec->devnull;
